/*
 * Copyright (c) 2006-2012 Rogério Liesenfeld
 * This file is subject to the terms of the MIT license (see LICENSE.txt).
 */
package mockit.coverage.modification;

import java.security.*;
import java.util.regex.*;

import mockit.coverage.standalone.*;

final class ClassSelection
{
   private final Matcher classesToInclude;
   private final Matcher classesToExclude;
   private final Matcher testCode;

   ClassSelection()
   {
      classesToInclude = getClassNameRegex("classes");
      classesToExclude = getClassNameRegex("excludes");
      testCode = Startup.isStandalone() ? null : Pattern.compile(".+Test(\\$.+)?").matcher("");
   }

   private Matcher getClassNameRegex(String propertySuffix)
   {
      String regex = System.getProperty("jmockit-coverage-" + propertySuffix, "");
      return regex.length() == 0 ? null : Pattern.compile(regex).matcher("");
   }

   boolean isSelected(String className, ProtectionDomain protectionDomain)
   {
      CodeSource codeSource = protectionDomain.getCodeSource();

      if (codeSource == null || className.charAt(0) == '[' || className.startsWith("mockit.")) {
         return false;
      }

      if (classesToExclude != null && classesToExclude.reset(className).matches()) {
         return false;
      }
      else if (classesToInclude != null && classesToInclude.reset(className).matches()) {
         return true;
      }
      else if (testCode == null || testCode.reset(className).matches()) {
         return false;
      }

      String location = codeSource.getLocation().getPath();

      return
         !location.endsWith(".jar") &&
         !location.endsWith("/test-classes/") && !location.endsWith("/jmockit/main/classes/");
   }
}
